import joinUrl from 'proper-url-join';
import objectToFormData from 'object-to-formdata';
import axios from 'axios';

const CancelToken = axios.CancelToken;
let cancel;

function Client(params){
    this.baseURL = params.baseURL;
    this.leadingSlash = params.leadingSlash;
    this.trailingSlash = params.trailingSlash;
}

Object.assign(Client.prototype, {
    send: function(method, url, data, headers) {
        let callURL = joinUrl(this.baseURL, url, {
            leadingSlash: this.leadingSlash,
            trailingSlash: this.trailingSlash,
            query: {'skip-basic-auth-header': true}
        });
        headers = headers || {};
        let body = '';

        if (method === 'POST' || method === 'PUT' || method === 'DELETE') {
            if (data instanceof FormData) {
                headers['Content-Type'] = 'multipart/form-data';
                body = data;
            } else {
                headers['Content-Type'] = 'application/json';
                body = data;
            }
        } else if (typeof data === 'object' && Object.keys(data).length && data.constructor === Object) {
            callURL = joinUrl(callURL, {trailingSlash: true, query: data});
        }

        this.events.onRequestStart(callURL, method, body, headers);
        return axios(callURL, {
            method,
            data: body,
            headers,
            cancelToken: new CancelToken(function executor(c) {
                cancel = c;
            }),
            validateStatus: function (status) {
                return status >= 200 && status <= 202;
            }
        })
            .then((response) => {
                this.events.onRequestComplete();
                this.events.onRequestSuccess(response);
                return response.data;
            })
            .catch((error) => {
                this.events.onRequestComplete();
                this.events.onRequestError(error.response);
                return Promise.reject(error.response);
            });
    },

    // use only to send files with additional params
    sendFormData: function(method, url, data, headers){
        return this.send(method, url, objectToFormData(data), headers);
    },
    cancel: function(){
        cancel();
    },
    events: {
        onRequestStart: function (url, requestOptions) {
        },
        onRequestSuccess: function (response) {
        },
        onRequestError: function (reason) {
        },
        onRequestComplete: function () {
        }
    }
});

export default Client;
